"""pygrowthmodels - A compilation of nonlinear growth models"""

from pygrowthmodels.blumberg import blumberg
from pygrowthmodels.blumberg import blumberg_inverse
from pygrowthmodels.brody import brody
from pygrowthmodels.brody import brody_inverse
from pygrowthmodels.chapman_richards import chapman_richards
from pygrowthmodels.chapman_richards import chapman_richards_inverse
from pygrowthmodels.gompertz import gompertz
from pygrowthmodels.gompertz import gompertz_inverse
from pygrowthmodels.janoschek import janoschek
from pygrowthmodels.janoschek import janoschek_inverse
from pygrowthmodels.logistic import generalised_logistic
from pygrowthmodels.logistic import generalised_logistic_inverse
from pygrowthmodels.logistic import logistic
from pygrowthmodels.logistic import logistic_inverse
from pygrowthmodels.mitcherlich import mitcherlich
from pygrowthmodels.mitcherlich import mitcherlich_inverse
from pygrowthmodels.mmf import mmf
from pygrowthmodels.mmf import mmf_inverse
from pygrowthmodels.monomolecular import monomolecular
from pygrowthmodels.monomolecular import monomolecular_inverse
from pygrowthmodels.negative_exponential import negative_exponential
from pygrowthmodels.negative_exponential import negative_exponential_inverse
from pygrowthmodels.richard import generalised_richard
from pygrowthmodels.richard import generalised_richard_inverse
from pygrowthmodels.richard import richard
from pygrowthmodels.richard import richard_inverse
from pygrowthmodels.schnute import schnute
from pygrowthmodels.schnute import schnute_inverse
from pygrowthmodels.stannard import stannard
from pygrowthmodels.stannard import stannard_inverse
from pygrowthmodels.vonbertalanffy import vonbertalanffy
from pygrowthmodels.vonbertalanffy import vonbertalanffy_inverse
from pygrowthmodels.weibull import weibull
from pygrowthmodels.weibull import weibull_inverse

__version__ = '0.0.1'
__author__ = u'Daniel Rodríguez Pérez <daniel.rodriguez.perez@gmail.com>'
__all__ = []



# -*- coding: utf8 -*-

"""Tests for the von Bertalanffy growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from pytest import approx

from pygrowthmodels import vonbertalanffy
from pygrowthmodels import vonbertalanffy_inverse


def test_vonbertalanffy():
    time = [0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0, 4.5]

    # Evaluate (1, 2, 3, 4) parameters
    value = [1.217769e+000, 1.035581e+000, 1.007518e+000, 1.001658e+000, 1.000369e+000, 1.000082e+000, 1.000018e+000,
             1.000004e+000, 1.000001e+000]

    for step in range(len(time)):
        assert vonbertalanffy(time[step], 1, 2, 3, 4) == approx(value[step])

    # Evaluate (12, 2, 3, -2) parameters
    value = [1.199897e+001, 1.199977e+001, 1.199995e+001, 1.199999e+001, 1.200000e+001, 1.200000e+001, 1.200000e+001,
             1.200000e+001, 1.200000e+001]

    for step in range(len(time)):
        assert vonbertalanffy(time[step], 12, 2, 3, -2) == approx(value[step])


def test_vonbertalanffy_inverse():
    assert vonbertalanffy_inverse(1.217769e+000, 1, 2, 3, 4) == approx(0.5)

    assert vonbertalanffy_inverse(1.200000e+001, 12, 2, 3, -2) == approx(float('inf'))



# -*- coding: utf8 -*-

"""Tests for the Stannard growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from pytest import approx

from pygrowthmodels import stannard
from pygrowthmodels import stannard_inverse


def test_stannard():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0, 4.0]

    # Evaluate (1, 2, 3, 0.5) parameters
    value = [0.01831257, 0.08180985, 0.34525776, 0.85501964, 0.99096609, 0.99954437, 0.9999773, 0.99999887, 0.99999994,
             1]

    for step in range(len(time)):
        assert stannard(time[step], 1, 2, 3, 0.5) == approx(value[step])

    # Evaluate (12, 2, 3, 0.7) parameters
    value = [0.2192816, 0.9660857, 3.7981941, 9.0791332, 11.5399613, 11.9437234, 11.9933643, 11.9992210, 11.9999086, 12]

    for step in range(len(time)):
        assert stannard(time[step], 12, 2, 3, 0.7) == approx(value[step])


def test_stannard_inverse():
    time = [-2.0, -1.5, -1.0, -0.5, 0.5, float('inf')]

    # Evaluate (1, 2, 3, 0.5) parameters
    value = [0.01831257, 0.08180985, 0.34525776, 0.85501964, 0.99954437, 1]

    for step in range(len(time)):
        assert stannard_inverse(value[step], 1, 2, 3, 0.5) == approx(time[step])

    # Evaluate (12, 2, 3, 0.7) parameters
    value = [0.2192816, 0.9660857, 3.7981941, 9.0791332, 11.9437234, 12]

    for step in range(len(time)):
        assert stannard_inverse(value[step], 12, 2, 3, 0.7) == approx(time[step])



# -*- coding: utf8 -*-

"""Tests for the von Weibull growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from pytest import approx

from pygrowthmodels import weibull
from pygrowthmodels import weibull_inverse


def test_weibull():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0]

    # Evaluate (1, 2, 3, 4) parameters
    value = [1, 9.999995e-001, 9.004259e-001, -6.580582e-001, -1, -6.580582e-001, 9.004259e-001, 9.999995e-001, 1]

    for step in range(len(time)):
        assert weibull(time[step], 1, 2, 3, 4) == approx(value[step])

    # Evaluate (1, 2, 3, 4) parameters
    value = [-7.948576e+002, -1.680343e+002, -2.817107e+001, 3.036622e+000, 10, 1.155374e+001, 1.190043e+001,
             1.197778e+001, 1.199504e+001]

    for step in range(len(time)):
        assert weibull(time[step], 12, 2, 3, 1) == approx(value[step])


def test_weibull_inverse():
    assert weibull_inverse(1, 1, 2, 3, 4) == approx(float('inf'))
    assert weibull_inverse(-1, 1, 2, 3, 4) == approx(0.0)
    assert weibull_inverse(-6.580582e-001, 1, 2, 3, 4) == approx(0.5)
    assert weibull_inverse(9.004259e-001, 1, 2, 3, 4) == approx(1.0)

    assert weibull_inverse(-7.948576e+002, 12, 2, 3, 1) == approx(-2.0)
    assert weibull_inverse(-1.680343e+002, 12, 2, 3, 1) == approx(-1.5)
    assert weibull_inverse(-2.817107e+001, 12, 2, 3, 1) == approx(-1.0)
    assert weibull_inverse(3.036622e+000, 12, 2, 3, 1) == approx(-0.5)
    assert weibull_inverse(10, 12, 2, 3, 1) == approx(0.0)
    assert weibull_inverse(1.155374e+001, 12, 2, 3, 1) == approx(0.5)



"""Stannard growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from math import exp
from math import log


def stannard(time, alpha, beta, rate, slope):
    """
    Computes the Stannard growth model

    Parameters
    ----------
    time : time
    alpha : upper asymptote
    beta : growth displacement
    rate : growth rate
    slope : slope of growth

    See Also
    --------
    stannard_inverse

    References
    ----------
    .. [1] A. Khamiz, Z. Ismail, and A. T. Muhammad, "Nonlinear growth models
           for modeling oil palm yield growth," Journal of Mathematics and
           Statistics, vol. 1, no. 3, p. 225, 2005.
    """

    result = alpha * (1 + exp(-(beta + rate * time) / slope)) ** (-slope)

    return result


def stannard_inverse(size, alpha, beta, rate, slope):
    """
    Computes the inverse of Stannard growth model

    Parameters
    ----------
    size : size
    alpha : upper asymptote
    beta : growth displacement
    rate : growth rate
    slope : slope of growth

    See Also
    --------
    stannard

    References
    ----------
    .. [1] A. Khamiz, Z. Ismail, and A. T. Muhammad, "Nonlinear growth models
           for modeling oil palm yield growth," Journal of Mathematics and
           Statistics, vol. 1, no. 3, p. 225, 2005.
    """

    if (alpha / size) ** (1 / slope) == 1:
        return float('inf')
    else:
        result = - (beta + slope * log((alpha / size) ** (1 / slope) - 1)) / rate

    return result



"""Brody growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from math import exp
from math import log


def brody(time, alpha, w0, rate):
    """
    Computes the Brody growth model

    Parameters
    ----------
    time : time
    alpha : upper asymptote
    w0 : a reference value at time = 0
    rate : growth rate

    See Also
    --------
    brody_inverse

    References
    ----------
    .. [1] M. M. Kaps, W. O. W. Herring, and W. R. W. Lamberson, "Genetic and
           environmental parameters for traits derived from the Brody growth
           curve and their relationships with weaning weight in Angus cattle.,"
           Journal of Animal Science, vol. 78, no. 6, pp. 1436-1442, May 2000.
    """

    result = alpha - (alpha - w0) * exp(- rate * time)

    return result


def brody_inverse(size, alpha, w0, rate):
    """
    Computes the inverse Brody growth model

    Parameters
    ----------
    size : size
    alpha : upper asymptote
    w0 : a reference value at time = 0
    rate : growth rate

    See Also
    --------
    brody

    References
    ----------
    .. [1] M. M. Kaps, W. O. W. Herring, and W. R. W. Lamberson, "Genetic and
           environmental parameters for traits derived from the Brody growth
           curve and their relationships with weaning weight in Angus cattle.,"
           Journal of Animal Science, vol. 78, no. 6, pp. 1436-1442, May 2000.
    """

    result = - log((alpha - size) / (alpha - w0)) / rate

    return result



# -*- coding: utf8 -*-

"""Tests for the Schnute growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from pytest import approx

from pygrowthmodels import schnute
from pygrowthmodels import schnute_inverse


def test_schnute():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0, 4.0]

    # Evaluate (1, 2, 3, 0.5) parameters
    value = [1.002476, 1.011048, 1.048606, 1.202606, 1.732051, 3.156482, 6.416469, 13.454897, 28.422836, 570.535348]

    for step in range(len(time)):
        assert schnute(time[step], 1, 2, 3, 0.5) == approx(value[step])

    # Evaluate (12, 2, 3, 0.2) parameters
    value = [1.643888, 1.644360, 1.646471, 1.655800, 1.695218, 1.837775, 2.205393, 2.862040, 3.825090, 12.662398]

    for step in range(len(time)):
        assert schnute(time[step], 12, 2, 3, 0.2) == approx(value[step])


def test_schnute_inverse():
    time = [1.0, 1.5, 2.0, 4.0]

    # Evaluate (1, 2, 3, 0.5) parameters
    value = [6.416469, 13.454897, 28.422836, 570.535348]

    for step in range(len(time)):
        assert schnute_inverse(value[step], 1, 2, 3, 0.5) == approx(time[step])

    # Evaluate (12, 2, 3, 0.2) parameters
    value = [2.205393, 2.862040, 3.825090, 12.662398]

    for step in range(len(time)):
        assert schnute_inverse(value[step], 12, 2, 3, 0.2) == approx(time[step])



# -*- coding: utf8 -*-

"""Tests for the Negative exponential growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#


from pytest import approx

from pygrowthmodels import negative_exponential
from pygrowthmodels import negative_exponential_inverse


def test_negative_exponential():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0]

    # Evaluate (1, 2) parameters
    value = [-5.359815e+001, -1.908554e+001, -6.389056e+000, -1.718282e+000, 0, 6.321206e-001, 8.646647e-001,
             9.502129e-001, 9.816844e-001]

    for step in range(len(time)):
        assert negative_exponential(time[step], 1, 2) == approx(value[step])

    # Evaluate (12, 1) parameters
    value = [-7.666867e+001, -4.178027e+001, -2.061938e+001, -7.784655e+000, 0, 4.721632e+000, 7.585447e+000,
             9.322438e+000, 1.037598e+001]

    for step in range(len(time)):
        assert negative_exponential(time[step], 12, 1) == approx(value[step])


def test_negative_exponential_inverse():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5]

    # Evaluate (1, 2) parameters
    value = [-5.359815e+001, -1.908554e+001, -6.389056e+000, -1.718282e+000, 0, 6.321206e-001, 8.646647e-001,
             9.502129e-001]

    for step in range(len(time)):
        assert negative_exponential_inverse(value[step], 1, 2) == approx(time[step])

    # Evaluate (12, 1) parameters
    value = [-7.666867e+001, -4.178027e+001, -2.061938e+001, -7.784655e+000, 0, 4.721632e+000, 7.585447e+000,
             9.322438e+000]

    for step in range(len(time)):
        assert negative_exponential_inverse(value[step], 12, 1) == approx(time[step])


def test_negative_exponential_inverse_nan():
    assert negative_exponential_inverse(10, 0, 2) == approx(float('nan'), nan_ok=True)
    assert negative_exponential_inverse(10, 1, 0) == approx(float('nan'), nan_ok=True)



"""Tests for the Chapman-Richards growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#


from pytest import approx

from pygrowthmodels import chapman_richards
from pygrowthmodels import chapman_richards_inverse


def test_chapman_richards():
    time = [0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0, 4.5]

    # Evaluate (1, 2, 3, 4) parameters
    value = [1.217769, 1.035581,1.007518,1.001658,1.000369,1.000082,1.000018,1.000004,1.000001]

    for step in range(len(time)):
        assert chapman_richards(time[step], 1, 2, 3, 4) == approx(value[step])


def test_chapman_richards_inverse():
    # Evaluate (1, 2, 3, 4) parameters
    assert chapman_richards_inverse(1.217769, 1, 2, 3, 4) == approx(0.5)



"""Tests for the Blumberg growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#


from pytest import approx

from pygrowthmodels import blumberg
from pygrowthmodels import blumberg_inverse


def test_blumberg():
    time = [0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0, 4.5]

    # Evaluate (1, 2, 3) parameters
    value = [0, 0.05882353, 0.33333333, 0.62790698, 0.8, 0.88652482, 0.93103448, 0.95543175, 0.96969697, 0.97852349]

    for step in range(len(time)):
        assert blumberg(time[step], 1, 2, 3) == approx(value[step])

    # Evaluate (10, 5, 0.43) parameters
    value = [0, 1.292630, 1.666667, 1.923072, 2.122552, 2.287412, 2.428652, 2.552605, 2.663284, 2.763410]

    for step in range(len(time)):
        assert blumberg(time[step], 10, 5, 0.43) == approx(value[step])

    # Evaluate (1, 2, 3, 2) parameters
    value = [0.8, 0.8865248, 0.9310345, 0.9554318, 0.9696970, 0.9785235, 0.9842520, 0.9881218, 0.9908257, 0.9927700]

    for step in range(len(time)):
        assert blumberg(time[step], 1, 2, 3, 2) == approx(value[step])

    # Evaluate (10, 5, 0.43, 2) parameters
    value = [2.122552, 2.287412, 2.428652, 2.552605, 2.663284, 2.763410, 2.854921, 2.939251, 3.017494, 3.090503]

    for step in range(len(time)):
        assert blumberg(time[step], 10, 5, 0.43, 2) == approx(value[step])


def test_blumberg_inverse():
    time = [0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0, 4.5]

    # Evaluate (1, 2, 3) parameters
    value = [0, 0.05882353, 0.33333333, 0.62790698, 0.8, 0.88652482, 0.93103448, 0.95543175, 0.96969697, 0.97852349]

    for step in range(len(time)):
        assert blumberg_inverse(value[step], 1, 2, 3) == approx(time[step])

    # Evaluate (10, 5, 0.43) parameters
    value = [0, 1.292630, 1.666667, 1.923072, 2.122552, 2.287412, 2.428652, 2.552605, 2.663284, 2.763410]

    for step in range(len(time)):
        assert blumberg_inverse(value[step], 10, 5, 0.43) == approx(time[step])

    # Evaluate (1, 2, 3, 2) parameters
    value = [0.8, 0.8865248, 0.9310345, 0.9554318, 0.9696970, 0.9785235, 0.9842520, 0.9881218, 0.9908257, 0.9927700]

    for step in range(len(time)):
        assert blumberg_inverse(value[step], 1, 2, 3, 2) == approx(time[step], rel=1e-5)



"""Tests for the Logistic growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#


from pytest import approx

from pygrowthmodels import generalised_logistic
from pygrowthmodels import generalised_logistic_inverse
from pygrowthmodels import logistic
from pygrowthmodels import logistic_inverse


def test_logistic():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0]

    # Evaluate (1, 2, 3) parameters
    value = [1.237842e-003, 5.523816e-003, 2.428890e-002, 1.003676e-001, 3.333333e-001, 6.914385e-001, 9.094430e-001,
             9.782649e-001, 9.950670e-001]

    for step in range(len(time)):
        assert logistic(time[step], 1, 2, 3) == approx(value[step])

    # Evaluate (12, 1, 2) parameters
    value = [2.158345e-001, 5.691105e-001, 1.430435e+000, 3.227297e+000, 6, 8.772703e+000, 1.056956e+001, 1.143089e+001,
             1.178417e+001]

    for step in range(len(time)):
        assert logistic(time[step], 12, 1, 2) == approx(value[step])


def test_logistic_inverse():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5]

    # Evaluate (1, 2, 3) parameters
    value = [1.237842e-003, 5.523816e-003, 2.428890e-002, 1.003676e-001, 3.333333e-001, 6.914385e-001, 9.094430e-001,
             9.782649e-001, 9.950670e-001]

    for step in range(len(time)):
        assert logistic_inverse(value[step], 1, 2, 3) == approx(time[step], abs=1e-6)

    # Evaluate (12, 1, 2) parameters
    value = [2.158345e-001, 5.691105e-001, 1.430435e+000, 3.227297e+000, 6, 8.772703e+000, 1.056956e+001, 1.143089e+001]

    for step in range(len(time)):
        assert logistic_inverse(value[step], 12, 1, 2) == approx(time[step], abs=1e-5)


def test_logistic_inverse_nan():
    assert logistic_inverse(0, 1, 2, 3) == approx(float('nan'), nan_ok=True)


def test_generalised_logistic():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0]

    # Evaluate (1, 3, 5, 2, 0) parameters
    value = [1.000045e+000, 1.000553e+000, 1.006715e+000, 1.078849e+000, 1.666667e+000, 2.717962e+000, 2.973407e+000,
             2.997790e+000, 2.999818e+000]

    for step in range(len(time)):
        assert generalised_logistic(time[step], 1, 3, 5, 2, 0) == approx(value[step])

    # Evaluate (12, 3, 5, 2, 1) parameters
    value = [1.200000e+001, 1.199998e+001, 1.199980e+001, 1.199751e+001, 1.196978e+001, 1.164518e+001, 9, 4.269170e+000,
             3.119670e+000]

    for step in range(len(time)):
        assert generalised_logistic(time[step], 12, 3, 5, 2, 1) == approx(value[step])


def test_generalised_logistic_inverse():
    # Evaluate (1, 3, 5, 2, 0) parameters
    assert generalised_logistic_inverse(1.078849e+000, 1, 3, 5, 2, 0) == approx(-0.5)
    assert generalised_logistic_inverse(2.717962e+000, 1, 3, 5, 2, 0) == approx(0.5)

    # Evaluate (12, 3, 5, 2, 1) parameters
    assert generalised_logistic_inverse(1.164518e+001, 12, 3, 5, 2, 1) == approx(0.5)
    assert generalised_logistic_inverse(9, 12, 3, 5, 2, 1) == approx(1.0)
    assert generalised_logistic_inverse(4.269170e+000, 12, 3, 5, 2, 1) == approx(1.5)
    assert generalised_logistic_inverse(3.119670e+000, 12, 3, 5, 2, 1) == approx(2.0)



# -*- coding: utf8 -*-

"""Tests for the Monomolecular growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#


from pytest import approx

from pygrowthmodels import monomolecular
from pygrowthmodels import monomolecular_inverse


def test_monomolecular():
    time = [-1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0]

    # Evaluate (1, 2, 3) parameters
    value = [-1.790343e+002, -3.917107e+001, -7.963378e+000, -1, 5.537397e-001, 9.004259e-001, 9.777820e-001,
             9.950425e-001]

    for step in range(len(time)):
        assert monomolecular(time[step], 1, 2, 3) == approx(value[step])

    # Evaluate (12, 1, 2) parameters
    value = [-2.290264e+002, -7.666867e+001, -2.061938e+001, 0, 7.585447e+000, 1.037598e+001, 1.140256e+001,
             1.178021e+001]

    for step in range(len(time)):
        assert monomolecular(time[step], 12, 1, 2) == approx(value[step])


def test_monomolecular_inverse():
    time = [-1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0]

    # Evaluate (1, 2, 3) parameters
    value = [-1.790343e+002, -3.917107e+001, -7.963378e+000, -1, 5.537397e-001, 9.004259e-001, 9.777820e-001,
             9.950425e-001]

    for step in range(len(time)):
        assert monomolecular_inverse(value[step], 1, 2, 3) == approx(time[step])


def test_monomolecular_inverse_nan():
    assert monomolecular_inverse(10, 1, 2, 3) == approx(float('nan'), nan_ok=True)



"""Janoschek growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from pytest import approx

from pygrowthmodels import janoschek
from pygrowthmodels import janoschek_inverse


def test_janoschek():
    time = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    # Evaluate (10, 2, 0.25, 2) parameters
    value = [2, 3.769594, 7.056964, 9.156806, 9.853475, 9.984556, 9.999013, 9.999962, 9.999999, 10]

    for step in range(len(time)):
        assert janoschek(time[step], 10, 2, 0.25, 2) == approx(value[step])


def test_janoschek_inverse():
    time = [0, 1, 2, 3, 4]

    # Evaluate (10, 2, 0.25, 2) parameters
    value = [2, 3.769594, 7.056964, 9.156806, 9.853475]

    for step in range(len(time)):
        assert janoschek_inverse(value[step], 10, 2, 0.25, 2) == approx(time[step])



"""Tests for the Gompertz exponential growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from pytest import approx

from pygrowthmodels import gompertz
from pygrowthmodels import gompertz_inverse


def test_gompertz():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0]

    # Evaluate (1, 2, 3) parameters
    value = [0, 6.488035e-079, 3.580340e-018, 1.280131e-004, 1.353353e-001, 6.400171e-001, 9.052228e-001, 9.780270e-001,
             9.950548e-001]

    for step in range(len(time)):
        assert gompertz(time[step], 1, 2, 3) == approx(value[step])

    # Evaluate (1, 2, 3) parameters
    value = [2.330805e-023, 2.270614e-008, 7.415748e-003, 7.918564e-001, 4.414553e+000, 8.306408e+000, 1.048108e+001,
             1.141718e+001, 1.178221e+001]

    for step in range(len(time)):
        assert gompertz(time[step], 12, 1, 2) == approx(value[step])


def test_gompertz_inverse():
    time = [-1.5, -1.0, -0.5, 0.5, 1.0, 1.5]

    # Evaluate (1, 2, 3) parameters
    value = [6.488035e-079, 3.580340e-018, 1.280131e-004, 6.400171e-001, 9.052228e-001, 9.780270e-001]

    for step in range(len(time)):
        assert gompertz_inverse(value[step], 1, 2, 3) == approx(time[step])


def test_gompertz_inverse_nan():
    assert gompertz_inverse(0, 1, 2, 3) == approx(float('nan'), nan_ok=True)



"""Blumberg growth model"""


#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#


def blumberg(time, alpha, w0, slope, time_0=0):
    """
    Computes the Blumberg growth model

    Parameters
    ----------
    time : time
    alpha : upper asymptote
    w0 : a reference value at time = time_0
    slope : slope of growth
    time_0 : time shift (default 0)

    See Also
    --------
    blumberg_inverse

    References
    ----------
    .. [1] A. Tsoularis and J. Wallace, "Analysis of logistic growth models.,"
           Math Biosci, vol. 179, no. 1, pp. 21-55, Jul. 2002.
    """

    result = (alpha * (time + time_0) ** slope) / (w0 + (time + time_0) ** slope)

    return result


def blumberg_inverse(size, alpha, w0, slope, time_0=0):
    """
    Computes the inverse of Blumberg growth model

    Parameters
    ----------
    size : size
    alpha: upper asymptote.
    w0: a reference value at time = time_0
    slope: slope of growth
    time_0 time shift (default 0)

    See Also
    --------
    blumberg

    References
    ----------
    .. [1] A. Tsoularis and J. Wallace, "Analysis of logistic growth models.,"
           Math Biosci, vol. 179, no. 1, pp. 21-55, Jul. 2002.
    """

    result = (size * w0 / (alpha - size)) ** (1 / slope) - time_0

    return result



"""Tests for the Janoschek growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from math import exp
from math import log


def janoschek(time, lower, alpha, rate, delta):
    """
        Computes the Janoschek growth model

        Parameters
        ----------
        time : time
        lower : lower asymptote
        alpha : upper asymptote
        rate : growth rate
        delta : the point of inflection.
    """

    result = lower - (lower - alpha) * exp(-rate * time ** delta)

    return result


def janoschek_inverse(size, lower, alpha, rate, delta):
    result = (log((lower - alpha) / (lower - size)) / rate) ** (1 / delta)

    return result



"""Tests for the Brody growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#


from pytest import approx

from pygrowthmodels import brody
from pygrowthmodels import brody_inverse


def test_brody():
    time = [0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0, 4.5]

    # Evaluate (1, 2, 3) parameters
    value = [2, 1.223130, 1.049787, 1.011109, 1.002479, 1.000553, 1.000123, 1.000028, 1.000006, 1.000001]

    for step in range(len(time)):
        assert brody(time[step], 1, 2, 3) == approx(value[step])

    # Evaluate (10, 5, 0.43) parameters
    value = [5, 5.967293, 6.747455, 7.376687, 7.884190, 8.293511, 8.623646, 8.889914, 9.104669, 9.277879]

    for step in range(len(time)):
        assert brody(time[step], 10, 5, 0.43) == approx(value[step])


def test_brody_inverse():
    time = [0.0, 0.5, 1.0, 1.5, 2.0, 2.5, 3.0, 3.5, 4.0, 4.5]

    # Evaluate (1, 2, 3) parameters
    value = [2, 1.223130, 1.049787, 1.011109, 1.002479, 1.000553, 1.000123, 1.000028, 1.000006, 1.000001]

    for step in range(len(time)):
        assert brody_inverse(value[step], 1, 2, 3) == approx(time[step], rel=1e-1)

    # Evaluate (10, 5, 0.43) parameters
    value = [5, 5.967293, 6.747455, 7.376687, 7.884190, 8.293511, 8.623646, 8.889914, 9.104669, 9.277879]

    for step in range(len(time)):
        assert brody_inverse(value[step], 10, 5, 0.43) == approx(time[step])



"""Tests for the Logistic growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#


from pytest import approx

from pygrowthmodels import mitcherlich
from pygrowthmodels import mitcherlich_inverse


def test_mitcherlich():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0]

    # Evaluate (1, 2, 3) parameters
    value = [7.777778e-001, 6.150998e-001, 3.333333e-001, -1.547005e-001, -1, -2.464102e+000, -5, -9.392305e+000, -17]

    for step in range(len(time)):
        assert mitcherlich(time[step], 1, 2, 3) == approx(value[step])

    # Evaluate (12, 1, 2) parameters
    value = [1.175000e+001, 1.164645e+001, 1.150000e+001, 1.129289e+001, 11, 1.058579e+001, 10, 9.171573e+000, 8]

    for step in range(len(time)):
        assert mitcherlich(time[step], 12, 1, 2) == approx(value[step])


def test_mitcherlich_inverse():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0]

    # Evaluate (1, 2, 3) parameters
    value = [7.777778e-001, 6.150998e-001, 3.333333e-001, -1.547005e-001, -1, -2.464102e+000, -5, -9.392305e+000, -17]

    for step in range(len(time)):
        assert mitcherlich_inverse(value[step], 1, 2, 3) == approx(time[step])


def test_mitcherlich_inverse_nan():
    assert mitcherlich_inverse(10, 1, 2, 3) == approx(float('nan'), nan_ok=True)



# -*- coding: utf8 -*-

"""Tests for the Richard growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#

from pytest import approx

from pygrowthmodels import generalised_richard
from pygrowthmodels import generalised_richard_inverse
from pygrowthmodels import richard
from pygrowthmodels import richard_inverse


def test_richard():
    assert richard(-2.0, 12, 2, 3, 1) == approx(1.485410e-002)
    assert richard(-1.5, 12, 2, 3, 1) == approx(6.628579e-002)
    assert richard(-1.0, 12, 2, 3, 1) == approx(2.914668e-001)
    assert richard(-0.5, 12, 2, 3, 1) == approx(1.204411e+000)
    assert richard(0.0, 12, 2, 3, 1) == approx(4)
    assert richard(0.5, 12, 2, 3, 1) == approx(8.297261e+000)
    assert richard(1.0, 12, 2, 3, 1) == approx(1.091332e+001)
    assert richard(1.5, 12, 2, 3, 1) == approx(1.173918e+001)
    assert richard(2.0, 12, 2, 3, 1) == approx(1.194080e+001)


def test_richard_inverse():
    assert richard_inverse(1.485410e-002, 12, 2, 3, 1) == approx(-2.0)
    assert richard_inverse(6.628579e-002, 12, 2, 3, 1) == approx(-1.5)
    assert richard_inverse(2.914668e-001, 12, 2, 3, 1) == approx(-1.0)
    assert richard_inverse(1.204411e+000, 12, 2, 3, 1) == approx(-0.5)
    assert richard_inverse(4, 12, 2, 3, 1) == approx(0.0)
    assert richard_inverse(8.297261e+000, 12, 2, 3, 1) == approx(0.5)
    assert richard_inverse(1.173918e+001, 12, 2, 3, 1) == approx(1.5)


def test_richard_nan():
    assert richard(-2.0, 1, -2, 3, 4) == approx(float('nan'), nan_ok=True)
    assert richard(-1.5, 1, -2, 3, 4) == approx(float('nan'), nan_ok=True)
    assert richard(-1.0, 1, -2, 3, 4) == approx(float('nan'), nan_ok=True)
    assert richard(-0.5, 1, -2, 3, 4) == approx(float('nan'), nan_ok=True)
    assert richard(0.0, 1, -2, 3, 4) == approx(float('nan'), nan_ok=True)


def test_generalised_richard():
    assert generalised_richard(-2.0, 1, 3, 5, 4, 2, 0) == approx(1.13804918)
    assert generalised_richard(-1.5, 1, 3, 5, 4, 2, 0) == approx(1.25789346)
    assert generalised_richard(-1.0, 1, 3, 5, 4, 2, 0) == approx(1.48143674)
    assert generalised_richard(-0.5, 1, 3, 5, 4, 2, 0) == approx(1.89119211)
    assert generalised_richard(0.0, 1, 3, 5, 4, 2, 0) == approx(2.51967137)
    assert generalised_richard(0.5, 1, 3, 5, 4, 2, 0) == approx(2.92542185)
    assert generalised_richard(1.0, 1, 3, 5, 4, 2, 0) == approx(2.99331824)
    assert generalised_richard(1.5, 1, 3, 5, 4, 2, 0) == approx(2.99944730)
    assert generalised_richard(2.0, 1, 3, 5, 4, 2, 0) == approx(2.99995460)

    assert generalised_richard(-2.0, 12, 4, 5, 1, 2, 1) == approx(11.99999878)
    assert generalised_richard(-1.5, 12, 4, 5, 1, 2, 1) == approx(11.99998509)
    assert generalised_richard(-1.0, 12, 4, 5, 1, 2, 1) == approx(11.99981840)
    assert generalised_richard(-0.5, 12, 4, 5, 1, 2, 1) == approx(11.99778827)
    assert generalised_richard(0.0, 12, 4, 5, 1, 2, 1) == approx(11.97313871)
    assert generalised_richard(0.5, 12, 4, 5, 1, 2, 1) == approx(11.68460462)
    assert generalised_richard(1.0, 12, 4, 5, 1, 2, 1) == approx(9.33333333)
    assert generalised_richard(1.5, 12, 4, 5, 1, 2, 1) == approx(5.12815137)
    assert generalised_richard(2.0, 12, 4, 5, 1, 2, 1) == approx(4.10637367)


def test():
    assert generalised_richard_inverse(1.13804918, 1, 3, 5, 4, 2, 0) == approx(-2.0)
    assert generalised_richard_inverse(1.25789346, 1, 3, 5, 4, 2, 0) == approx(-1.5)
    assert generalised_richard_inverse(1.48143674, 1, 3, 5, 4, 2, 0) == approx(-1.0)
    assert generalised_richard_inverse(1.89119211, 1, 3, 5, 4, 2, 0) == approx(-0.5)
    assert generalised_richard_inverse(2.92542185, 1, 3, 5, 4, 2, 0) == approx(0.5)
    assert generalised_richard_inverse(2.99331824, 1, 3, 5, 4, 2, 0) == approx(1.0)
    assert generalised_richard_inverse(2.99944730, 1, 3, 5, 4, 2, 0) == approx(1.5)

    assert generalised_richard_inverse(11.99778827, 12, 4, 5, 1, 2, 1) == approx(-0.5)
    assert generalised_richard_inverse(11.68460462, 12, 4, 5, 1, 2, 1) == approx(0.5)
    assert generalised_richard_inverse(9.33333333, 12, 4, 5, 1, 2, 1) == approx(1.0)
    assert generalised_richard_inverse(5.12815137, 12, 4, 5, 1, 2, 1) == approx(1.5)
    assert generalised_richard_inverse(4.10637367, 12, 4, 5, 1, 2, 1) == approx(2.0)



import setuptools

setuptools.setup(
    name="pygrowthmodels",
    version="0.0.2",
    url="https://github.com/drodriguezperez/pygrowthmodels.git",

    author=u"Daniel Rodríguez Pérez",
    author_email="daniel.rodriguez.perez@gmail.com",

    license='GPL-3',

    description="A compilation of nonlinear growth models",
    long_description=open('README.rst').read(),

    packages=setuptools.find_packages(),

    install_requires=[],

    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



# -*- coding: utf8 -*-

"""Tests for the Morgan-Mercer-Flodin growth model"""

#
#  Created by Daniel Rodríguez Pérez.
#
#  Copyright (c) 2018 Daniel Rodríguez Pérez.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#


from pytest import approx

from pygrowthmodels import mmf
from pygrowthmodels import mmf_inverse


def test_mmf():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0, 3.0]

    # Evaluate (1, 2, 3, 4) parameters
    value = [1.157895, 1.372093, 1.750000, 1.979592, 2.000000, 1.979592, 1.750000, 1.372093, 1.157895, 1.035714]

    for step in range(len(time)):
        assert mmf(time[step], 1, 2, 3, 4) == approx(value[step])

    # Evaluate (12, 2, 3, 1) parameters
    value = [-18, -8, -3, 0, 2, 3.428571, 4.5, 5.333333, 6, 7]

    for step in range(len(time)):
        assert mmf(time[step], 12, 2, 3, 1) == approx(value[step])


def test_mmf_inverse():
    time = [-2.0, -1.5, -1.0, -0.5, 0.0, 0.5, 1.0, 1.5, 2.0, 3.0]

    # Evaluate (1, 2, 3,4) parameters
    assert mmf_inverse(2.000000, 1, 2, 3, 4) == approx(0.0)
    assert mmf_inverse(1.750000, 1, 2, 3, 4) == approx(1.0)
    assert mmf_inverse(1.372093, 1, 2, 3, 4) == approx(1.5)
    assert mmf_inverse(1.157895, 1, 2, 3, 4) == approx(2.0)

    # Evaluate (12, 2, 3, 1) parameters
    value = [-18, -8, -3, 0, 2, 3.428571, 4.5, 5.333333, 6, 7]

    for step in range(len(time)):
        assert mmf_inverse(value[step], 12, 2, 3, 1) == approx(time[step])
